/*
 * Copyright (c) 2012 Chabot Simon
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS 0AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef EMITTER_7EKK7FLO
#define EMITTER_7EKK7FLO

#include <vector>
#include "Vector.h"
#include "Particle.h"

/**
 * \file Emitter.h
 * \author Simon Chabot <simon.chabot@etu.utc.fr>
 **/

/**
 * \class Emitter
 * \brief This class define a emitter, used to throw particles of fluide, if some particles are
 * missing
 */

class Emitter {
  public :
    /**
     * \brief Constructor
     * \param pos : position of the emitter
     * \param dir : direction of run
     * \param speed : fluid particles' initial speed
     * \param size : size of the emitter (controls the number of particles thrown at each step)
     * \param isOn : is the emitter active ?
     **/
    Emitter(const Vector &pos = Vector(), const Vector &dir = Vector(1, 1, 1), float speed = 0.5, float size = 1, bool isOn = true);

    /**
     * \fn emit(int &N, std::vector<Particle> &particles, int max)
     * \brief Emits some particles
     * \param N : current number of particle in the simulation (N is updated after this called)
     * \param particles : simulation's particles storage
     * \param max : maximum number of particles used for the simulation
     **/
    void emit(int &N, std::vector<Particle> &particles, int max);

    /**
     * \fn toggle()
     * \brief Toggles the emitter on/off
     **/
    void toggle() { _isOn = !_isOn ; };
     
  private :
    Vector _pos,
           _direction;

    float _speed;
    float _size;
    bool _isOn;

};

#endif /* end of include guard: EMITTER_7EKK7FLO */
