/*
 * Copyright (c) 2012 Chabot Simon
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS 0AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef FEEDBACK_S3FXUXNS
#define FEEDBACK_S3FXUXNS

#include "Particle.h"
#include "Parameters.h"
#include <vector>

/**
 * \file Feedback.h
 * \author Simon Chabot <simon.chabot@etu.utc.fr>
 **/

/**
 * \class Feedback
 * \brief It makes the computation needed for the feedback acceleration
 **/

class Feedback {
  public :

    /**
     * \brief Constructor
     * \param isOn : active or not ?
     **/
    Feedback(bool isOn = true);
    
    /**
     * \brief Compution the surface elevation
     * \param h : surface elevation storage
     * \param x : place where surface elevation needs to be computed
     * \param particles : simulation's particles
     * \param pm : simulation's parameters
     */
    void computeHeigths(std::vector<float> &h, const std::vector<float> &x, const std::vector<Particle> &particles, const Parameters &pm);

    /**
     * \brief Compute the acceleration which need to be applied to reduce the surface oscillations
     * \param h : surface elevation (computed by \fn computeHeigths())
     * \param pm : simulation's parameters (feedback acceleration is stored in pm)
     */
    void computeAcc(const std::vector<float> &h, Parameters &pm);


    /**
     * \brief Toggles whether the feedback is active or not
     */
    void toggle() { _isOn = !_isOn; }

    bool isOn() const { return _isOn; }

  private :
    bool _isOn;
};



#endif /* end of include guard: FEEDBACK_S3FXUXNS */
