/*
 * Copyright (c) 2012 Wicke Martin, Chabot Simon
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS 0AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef PARAMETERS_AAZDPGR8
#define PARAMETERS_AAZDPGR8

#include "Vector.h"
#include "Emitter.h"

class ContainerBox;

/**
 * \struct Parameters
 * \brief This structure defines the parameters needed for the sph computation
 */
struct Parameters {
  float simscale;             /*! Simulation scale                                     */
  float dt;                   /*! time step                                            */
  float rho0;                 /*! Rest density of the fluid                            */
  float intstiff;             /*! Constrols the fluid stiffness                        */ 
  float visc;                 /*! Viscosity parameters                                 */
  float pressureCorrection;   /*! Coefficient used in case of negative pressure        */
  Vector gravity;             /*! Gravity vector                                       */
  Vector acc;                 /*! Internal acceleration                                */
  Vector feedback;            /*! Feedback acceleration                                */
  Vector pressureResultant;   /*! Pressure resultant force                             */
  int N;                      /*! Number of particles used for the simulation          */
  int numberOfNeighbors;      /*! Number of neighbors on average for one particle      */
  float particle_radius;      /*! Particle radius (computed by SPH                     */
  ContainerBox *domain;          /*! Container where the fluid evolves                    */
  ContainerBox *fluid_init;      /*! Container where the fluid is initialy distributed    */
  Emitter *emitter;           /*! Emitter used to fill the domain if needed            */
  float maxvel;               /*! Velocity limiting                                    */
};



#endif /* end of include guard: PARAMETERS_AAZDPGR8 */
