#include <GL/glut.h>
#include <stdio.h>
#include "Plotlib.h"
#include "constantes.h" //We need window's width and height
#include "gl_helper.h"

Plot::Plot(const Vector &origin, const Vector &max) :
 _O(origin),
 _max(max),
 _N(200) {

  computeScale();
}

void Plot::computeScale() {
  //W and H can be found in constantes.h
  //Full width and half height;

  _scale.x = (WIDTH - 2 * _O.x) / _max.x; 
  _scale.y = (HEIGHT) / (3 * _max.y); 

  _d.x = _max.x / _N;
  _d.y = 3 * _max.y / _N;
}

void Plot::setMax(const Vector &max) {
  _max = max;
  computeScale();
}

bool Plot::isRecordable(const Vector &p) const {
  return (_points.size() == 0 || (p.x - _points.back().x >= _d.x));
}

bool Plot::isRecordable(float x, float y) const {
  return (_points.size() == 0 || (x - _points.back().x >= _d.x));
}

void Plot::addPoint(float x, float y) {
  if(isRecordable(x, y)) {
    if(x > _max.x) {
      _max.x = x;
      computeScale();
    }

    if(y > _max.y) {
      _max.y = y;
      computeScale();
    }

    _points.push_back(Vector(x, y));
  }
}

void Plot::plot(float R, float G, float B) const {
  std::vector<Vector>::const_iterator p(_points.begin());

  glColor3f(R, G, B);
  glBegin(GL_LINE_STRIP);
    while(p != _points.end()) {
      glVertex2f(_O.x + p->x * _scale.x, _O.y - p->y * _scale.y);
      p++;
    }
  glEnd();
}

void Plot::plotAxis(float R, float G, float B, bool yAxisOnLeft) const {
  plotXAxis(R, G, B);
  plotYAxis(R, G, B, yAxisOnLeft);
}

void Plot::plotXAxis(float R, float G, float B) const {
  int i = 1;
  int s;

  glColor3f(R, G, B);
  glBegin(GL_LINES);
    glVertex2f(_O.x, _O.y);
    glVertex2f(_O.x + _max.x * _scale.x, _O.y);

    while(2 * i * _d.x < _max.x) {
      s = ((2 * i) % 10 == 0) ? 4 : 2;
      glVertex2f(_O.x + 2 * i * _d.x * _scale.x, _O.y - s);
      glVertex2f(_O.x + 2 * i * _d.x * _scale.x, _O.y + s);

      i++;
    }

    /* Let's draw a beautiful arrow */
    glVertex2f(_O.x + _max.x * _scale.x - 5, _O.y - 3);
    glVertex2f(_O.x + _max.x * _scale.x, _O.y);
    glVertex2f(_O.x + _max.x * _scale.x - 5, _O.y + 3);
    glVertex2f(_O.x + _max.x * _scale.x, _O.y);
  glEnd();

  i = 1;
  char disp[10];
  while(10 * i * _d.x < _max.x) {
    sprintf(disp, "%2.3f", 10 * i * _d.x);
    drawText(_O.x + 10 * i * _d.x * _scale.x - 10, _O.y + 15, disp);
    i++;
  }
}
    

void Plot::plotYAxis(float R, float G, float B, bool onLeft) const {
  int i = 1;
  int s;
  float marge = onLeft ? 0 : _max.x * (0.5 * _scale.x);

  glColor3f(R, G, B);
  glBegin(GL_LINES);
      while(4 * i * _d.y < _max.y) {
        s = ((4 * i) % 8 == 0) ? 4 : 2;
        glVertex2f(_O.x + marge  - s, _O.y - 4 * i * _d.y * _scale.y);
        glVertex2f(_O.x + marge + s, _O.y - 4 * i * _d.y * _scale.y);
        i++;
      }

      glVertex2f(_O.x + marge, _O.y);
      glVertex2f(_O.x + marge, _O.y - _max.y * _scale.y);

      /* And an other one */
      glVertex2f(_O.x + marge - 3, _O.y - _max.y * _scale.y + 5);
      glVertex2f(_O.x + marge , _O.y - _max.y * _scale.y);
      glVertex2f(_O.x + marge + 3, _O.y - _max.y * _scale.y + 5);
      glVertex2f(_O.x + marge, _O.y - _max.y * _scale.y);
  glEnd();

  i = 1;
  char disp[10];
  while(8 * i * _d.y < _max.y) {
    sprintf(disp, "%2.3f", 8 * i * _d.y);
    drawText(_O.x + marge + 8, _O.y - (8 * i * _d.y * _scale.y) + 3, disp);
    i++;
  }
}
