/*
 * Copyright (c) 2012 Chabot Simon
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS 0AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef PLOTLIB_H7ST9I2M
#define PLOTLIB_H7ST9I2M

#include <vector>
#include "Vector.h"

/**
 * \file Plotlib.h
 * \author Simon Chabot <simon.chabot@etu.utc.fr>
 **/


/**
 * \class Plot
 **/


class Plot {

  public :

    /**
     * \brief Define the plot size
     * \param origin : point used for the origin (in global coordinates)
     * \param max    : the farest point which can be on the graph
     *
     * ^                                 max 
     * |
     * |
     * |
     * |
     * origin ---------------------------->
     */
    
    Plot(const Vector &origin, const Vector &max = Vector());

    /**
     * \brief Update the scale
     **/
    void computeScale();

    /**
     * \brief Set the farest point
     * \param max
     **/
    void setMax(const Vector &max);

    /**
     * \brief Add a point on the graph (added only if possible)
     * \param x : abscissa of the point
     * \param y : ordinate of the point
     **/
    void addPoint(float x, float y);
    void addPoint(const Vector &p) { addPoint(p.x, p.y); }

    /**
     * \brief Plot the current graph
     * \param RGB : color used for plotting
     **/
    void plot(float R = 1, float G = 1, float B = 1) const;

    /**
     * \brief Plot both axis
     * \param RGB : color
     * \param yAxisOnLeft : is y-axis on the left hand side, or the middle ?
     **/
    void plotAxis(float R = 1, float G = 1, float B = 1, bool yAxisOnLeft = true) const;

    /**
     * \brief Plot x-axis
     * \param RGB : color
     **/
    void plotXAxis(float R = 1, float G = 1, float B = 1) const;

    /**
     * \brief Plot y-axis
     * \param RGB : color
     * \param onLeft : is y-axis on the left hand side, or the middle ?
     **/
    void plotYAxis(float R = 1, float G = 1, float B = 1, bool onLeft = true) const;

  private :

    /**
     * \brief Is this point recordable ? ie, not so closed from the last one ?
     * \param xy : coordinates of the point to test
     **/
    bool isRecordable(float x, float y) const;
    bool isRecordable(const Vector &p) const;

    std::vector<Vector> _points;
    
    Vector _O; // Origin;
    Vector _max; // Used for scale computing

    Vector _scale;

    Vector _d; //Intervale minimun entre deux mesures.
    const int _N;
};




#endif /* end of include guard: PLOTLIB_H7ST9I2M */
