#include <GL/glut.h>
#include <fstream>
#include <string>
#include <cstdlib>
#include <utility>
#include <corona.h>
#include "Parameters.h"
#include "Sph.h"
#include "Container.h"
#include "Emitter.h"
#include "Feedback.h"
#include "Plotlib.h"
#include "gl_helper.h"
#include "helper.h"
#include "constantes.h"

#define DEG2RAD(d) (d * M_PI / 90.0f)

typedef struct {
  Vector from;
  Vector to;
  Vector ang; //Coordonnées sphériques (x <--> psi, y <--> phi, z <--> r)
  float fov;
} Direction;

Direction cam;

float view_matrix[16],
      model_matrix[16],
      proj_matrix[16];

bool drawTree(false), drawParticles(true), drawPseudoSurface(false), running(false), start(false),
     savePicture(false);

int winid(0);
std::vector<float> h, xh; //h(i) : hauteur du fluide dans le plan x = xh(i)

ContainerBox domain_b(40000, 256, Vector(-26.6, -16.6, 0), Vector(26.6, 16.6, 40), true),
             fluid_b (0, 0,       Vector(-26.6, -16.6, 0), Vector(26.6, 16.6, 20));

Emitter emitter(Vector(35, 0, 25), Vector(-1, 0, -0.3), 6, 4);

std::string curr_line;
std::ifstream data_file("log/demo.foo");
Vector speed_theo, speed_actual;
Plot speed_theo_plot(Vector(25, HEIGHT - 50)), 
     speed_actual_plot(Vector(25, HEIGHT - 50)),
     eta_plot[2] = {Plot(Vector(25, HEIGHT - 50)), Plot(Vector(25, HEIGHT - 50))};

float foo_time(0), bar_time(0), baz_time(0), max_time(0);

Parameters pm = {
  0.003,               // Unit size
  1.0E-3,              // dt --> time step
  1000,                // rest density
  2.93,                 // stiffness parameters 
  -1.0E-6,              // viscosity
  0.8,                 // Negatif pressure coeff
  Vector(0, 0, -9.81), // gravity
  Vector(0, 0, 0),     // acc
  Vector(0, 0, 0),     // feedback acc
  Vector(0, 0, 0),     //Pressure resultant force
  15000,               // N
  50,                  // Number of neighbors
  0,                   // particle radius (computed by SPH)
  &domain_b,
  &fluid_b,
  &emitter,
  200,                 //Max velocity
};

SPH sph(pm, "save.tx00");
Feedback feedback(true);
//SPH sph(pm);
//SPH sph(pm, "save_15000.tx00");

void exportPicture() {
  const int width = 1280,
            height = 800,
            bit_r = 16,
            bit_g = 16,
            bit_b = 16,
            bit_a = 0;

  const float DT = sph.getParams().dt * 3;
  static float lastSave = bar_time;
  static const int SESSION_ID = time(NULL);

  if(lastSave + 10*DT > bar_time)
    return;

  lastSave = bar_time;

  GLvoid *imageData = malloc(width*height*(bit_r + bit_g + bit_b + bit_a)); //Allocate memory for storing the image

  glReadPixels(0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, imageData); //Copy the image to the array imageData

  corona::Image *myImage = corona::CreateImage(width, height, corona::PF_R8G8B8A8, imageData);

  corona::FlipImage(myImage, corona::CA_X); //Flip the image

  static int i = 0;
  char name[200];
  sprintf(name, "images/img-%d-%05d.png", SESSION_ID, i++);
  corona::SaveImage(name, corona::FF_PNG, myImage); //Save it

  delete myImage;
  free(imageData);
}

   

void set_acc(std::string &line) {
  Vector &acc = sph.getParams().acc;
  float x, y, z;

  sscanf(line.c_str(), "%f;%f;%f;%f", &foo_time, &x, &y, &z);

  acc.x = x;
  acc.y = y;
  acc.z = z;
}

void update_cam() {
  float r = cam.ang.z,
        psi = DEG2RAD(cam.ang.x),
        phi = DEG2RAD(cam.ang.y);

  cam.from.x = cam.to.x + r*cos(psi)*cos(phi);
  cam.from.y = cam.to.y + r*cos(psi)*sin(phi);
  cam.from.z = cam.to.y + r*sin(psi);
}

void computeProjection() {
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(cam.fov, WIDTH/HEIGHT, 10.0, 800.0);
  glPushMatrix();
  glGetFloatv(GL_MODELVIEW_MATRIX, proj_matrix);
  glPopMatrix();
}

void computeView() {
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(cam.from.x,  cam.from.y,  cam.from.z,
            cam.to.x  ,  cam.to.y  ,  cam.to.z,
            0         ,  0         ,  1);
  glPushMatrix();
  glGetFloatv(GL_MODELVIEW_MATRIX, view_matrix);
  glPopMatrix();
}


void drawScene(float *viewmat) {
  glEnable(GL_LIGHTING);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHT0);

  GLfloat diff[4], spec[4], shininess = 60.0;

  diff[0] = 0.8f; diff[1] = 0.8f; diff[2] = 0.8f; diff[3] = 1.0f;
  spec[0] = 1.0f; spec[1] = 1.0f; spec[2] = 1.0f; spec[3] = 1.0f;

  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, &diff[0]);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, &spec[0]);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, &shininess);
  glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

  glColor3f(1, 1, 1);
  glLoadMatrixf(viewmat);
  glBegin(GL_QUADS);
    glNormal3f(0, 0, 1);
    glVertex3f(-1000, -1000, -0.1);
    glVertex3f(1000, -1000, -0.1);
    glVertex3f(1000, 1000,  -0.1);
    glVertex3f(-1000, 1000, -0.1);
  glEnd();

  sph.getParams().domain->draw();
  sph.getParams().fluid_init->draw();
  
  if (drawPseudoSurface) {
    glBegin(GL_LINES);
      for(int i = 0 ; i < h.size() ; i++) {
        glVertex3f(xh[i], domain_b.bottom().y, h[i]); 
        glVertex3f(xh[i], domain_b.top().y, h[i]); 
      }
    glEnd();
  }

  if (drawParticles)
    sph.drawParticles(viewmat);

  if (drawTree)
    sph.tree().draw();
}

void displaySpeedometersAndAcc() {
  char disp[200];
  glColor4f(1.0, 1.0, 1.0, 1.0);
  float x0 = 20,
        y0 = 40,
        dy = 20,
        i = 0;

  /** Draw acc variations **/
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  glBegin(GL_TRIANGLES); 

    glColor4f(0.8, 0.2, 0.2, 1.0);
    glVertex2f(x0 + 50, y0 + dy * i);
    glVertex2f(x0 + 50 + (sph.getParams().acc.x + sph.getParams().pressureResultant.x + sph.getParams().feedback.x) * 20,  y0 + dy * i + dy * 0.5);
    glVertex2f(x0 + 50, y0 + dy * i + dy);
    i += 1;

    glColor4f(0.2, 0.2, 0.8, 1.0);
    glVertex2f(x0 + 50, y0 + dy * i);
    glVertex2f(x0 + 50 + sph.getParams().acc.x * 20, y0 + dy * i + dy * 0.5);
    glVertex2f(x0 + 50, y0 + dy * i + dy);
    i += 2;
  glEnd();
  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

  glColor4f(0.8, 0.2, 0.2, 1.0);
  sprintf(disp, "Actual speed : %3.3f km/h", 
    3.6 * speed_actual.norm() * (speed_actual.x >= 0 ? 1 : -1));
  drawText(x0, y0 + dy * i++, disp);

  glColor4f(0.2, 0.2, 0.8, 1.0);
  sprintf(disp, "Theo speed : %3.3f km/h",
    3.6 * speed_theo.norm() * (speed_theo.x >= 0 ? 1 : -1));
  drawText(x0, y0 + dy * i++, disp);

  glColor4f(1, 1, 1, 1.0);
  sprintf(disp, "Actual acc : [%3.3fm/s^2, %3.3fm/s^2]", 
    sph.getParams().acc.x + sph.getParams().feedback.x + sph.getParams().pressureResultant.x, 
    sph.getParams().acc.y + sph.getParams().feedback.y + sph.getParams().pressureResultant.y);
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Fixed acc : [%3.3fm/s^2, %3.3fm/s^2]", sph.getParams().acc.x, sph.getParams().acc.y); 
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Feedback acc : [%3.3fm/s^2]", sph.getParams().feedback.x);
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Pressure resultant acc: [%3.3fm/s^2]", sph.getParams().pressureResultant.x); 
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Surface elevation :"); 
  drawText(x0, y0 + dy * i++, disp);
    glColor3f(0, 1, 1);
    sprintf(disp, "Front : %1.3fm", 
      (h[0] - sph.getParams().fluid_init->top().z) * sph.getParams().simscale); 
    drawText(x0 + dy, y0 + dy * i++, disp);

    glColor3f(1, 1, 0);
    sprintf(disp, "Back : %1.3fm",
      (h[1] - sph.getParams().fluid_init->top().z) * sph.getParams().simscale); 
    drawText(x0 + dy, y0 + dy * i++, disp);
}

void displaySimulationState() {

  float x0 = WIDTH - 170,
        y0 = 100,
        dy = 20,
        i = 0;


  char disp[200];
  glColor4f(1.0, 1.0, 1.0, 1.0);

  sprintf(disp, "Number of particles : %d", sph.getNb());
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Time : %lf %lf", baz_time + foo_time, bar_time);
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Running (space): %s", ONOFF(running));
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Fluid structure int. (P) : %s", ONOFF(sph.getParams().domain->pressureFieldCompution()));
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Feedback (F) : %s", ONOFF(feedback.isOn()));
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Save particles' position (J)");
  drawText(x0, y0 + dy * i++, disp);

  sprintf(disp, "Export pictures (E) : %s", ONOFF(savePicture));
  drawText(x0, y0 + dy * i++, disp);
}

void draw2DOverlay() {
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glScalef(2.0/WIDTH, -2.0/HEIGHT, 1);
  glTranslatef(-WIDTH/2.0, -HEIGHT/2.0, 0.0);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glPushMatrix();
    glGetFloatv(GL_MODELVIEW_MATRIX, view_matrix);
  glPopMatrix();

  displaySpeedometersAndAcc();
  displaySimulationState();


  /* Graphs plotting */
  speed_theo_plot.plotAxis();
  eta_plot[0].plotYAxis(0, 0.8, 0, false);

  speed_theo_plot.plot(0, 0, 1);
  speed_actual_plot.plot(1, 0, 0);

  eta_plot[0].plot(0, 1, 1);
  eta_plot[1].plot(1, 1, 0);
}


void display() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glShadeModel(GL_SMOOTH);

  computeProjection();
  computeView();


  glEnable(GL_LIGHTING);
  glLoadMatrixf(view_matrix);
  drawScene(view_matrix);
  draw2DOverlay();


  glutSwapBuffers();
}

void reshape(int w, int h) {
  glViewport(0, 0, (GLsizei) w, (GLsizei) h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, 1, 0, 1, -1, -1);
  glFrustum(-1, 1, -1, 1, 1.5, 20);
  glMatrixMode(GL_MODELVIEW);
}

void idle() {
  if (running) {

    if(bar_time >= (foo_time + baz_time) && sph.getNb() == pm.N && start) {
      if(!getline(data_file, curr_line))
        sph.getParams().acc = Vector();
      else
        set_acc(curr_line);
    } else if(sph.getNb() != pm.N || !start)
      baz_time += sph.getParams().dt;

    bar_time += sph.getParams().dt;

    speed_theo += sph.getParams().acc * sph.getParams().dt;
    speed_actual += (sph.getParams().acc + sph.getParams().pressureResultant + sph.getParams().feedback) * sph.getParams().dt;

    sph.step();

    feedback.computeHeigths(h, xh, sph.getParticles(), sph.getParams());
    feedback.computeAcc(h, sph.getParams());

    speed_theo_plot.addPoint(bar_time, speed_theo.x * 3.6);
    speed_actual_plot.addPoint(bar_time, speed_actual.x * 3.6);

    eta_plot[0].addPoint(bar_time, h[0] * sph.getParams().simscale);
    eta_plot[1].addPoint(bar_time, h[1] * sph.getParams().simscale);

    if(savePicture) {
      glDrawBuffer(GL_BACK);
      display();
      glReadBuffer(GL_BACK);
      exportPicture();
    }
    else 
      glutPostRedisplay();

    if(bar_time > max_time) {
      std::cout << "Simulation is over." << std::endl;
      running = false;
      savePicture = false;
    }
  }
}

void keyboard(unsigned char key, int x, int y) {
  bool cam_has_moved = false;
  bool redisplay = true;

  switch (key) {
    case 'P' : sph.getParams().domain->togglePressureFieldComputation(); break;
    case ' ' : running = !running; break;
    case '_' : drawPseudoSurface = !drawPseudoSurface; break;
    case 'I' : emitter.toggle(); break;
    case 'E' : savePicture = !savePicture; break;
    case 'd' : drawParticles = !drawParticles; break;
    case 'T' : drawTree = !drawTree; break;
    case 'S' : sph.getParams().acc.reset(); break;
    case 'A' : start = true; break;
    case 'j' : sph.save(); break;
    case 'F' : feedback.toggle(); sph.getParams().feedback.reset(); break;
    

    /* Gestiton de la caméra */

      //Déplacement du point d'origine
      //
      //Coordonnées sphériques :
      //  x <--> psi
      //  y <--> phi
      //  z <--> r
    case 'a' : //Turning left
      cam.ang.x -= 3;
      if (cam.ang.x < -90) cam.ang.x = -90;
      cam_has_moved = true;
      break;

    case 'e' : //Turning right
      cam.ang.x += 3;
      if (cam.ang.x > 90) cam.ang.x = 90;
      cam_has_moved = true;
      break;

    case 'u' : //Turning “up”
      cam.ang.y += 3;
      if (cam.ang.y > 180) cam.ang.y = 180;
      cam_has_moved = true;
      break;
    
    case 'i' : //Turning “down”
      cam.ang.y -= 3;
      if (cam.ang.y < 0) cam.ang.y = 0;
      cam_has_moved = true;
      break;

    case 'b' : 
      cam.ang.z += 5; 
      cam_has_moved = true;
      break;
    case 'p' : 
      cam.ang.z -= 5; 
      cam_has_moved = true;
      break;


      //Déplacement du point fixé
    case 'c' :
      cam.to.y += 5;
      cam_has_moved = true;
      break;

    case 'r' :
      cam.to.y -= 5;
      cam_has_moved = true;
      break;

    case 't' :
      cam.to.x -= 5;
      cam_has_moved = true;
      break;

    case 's' :
      cam.to.x += 5;
      cam_has_moved = true;
      break;

    case 'C' :
      cam.to.z += 5;
      cam_has_moved = true;
      break;

    case 'R' :
      cam.to.z -= 5;
      cam_has_moved = true;
      break;


    case 'q' :
      redisplay = false;
      glutDestroyWindow(winid);
      break;

    default : break;
  }

  if(cam_has_moved) {
    update_cam();
    std::cout << "Cam.from : " << cam.from << std::endl;
    std::cout << "Cam.to : " << cam.to << std::endl;
    std::cout << "Phi = " << cam.ang.x << ", Psi = " << cam.ang.y << ", r = " << cam.ang.z << std::endl;
  }

  if(redisplay)
    glutPostRedisplay();
}

void keyboard_special(int key, int x, int y) {
  Vector d(0, 0, 0);

  switch(key) {
    case GLUT_KEY_UP :   d.x = 0.5;   break;
    case GLUT_KEY_DOWN : d.x = -0.5;  break;
    case GLUT_KEY_LEFT : d.y = 0.5;   break;
    case GLUT_KEY_RIGHT : d.y = -0.5; break;
    default : break;
  }

  sph.getParams().acc += d;
}


void init() {
  glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
  glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);

  glClearColor(0.49, 0.49, 0.49, 1.0);
  glShadeModel(GL_SMOOTH);

  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glDepthMask(1);

  //Callbacks
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboard);
  glutSpecialFunc(keyboard_special);
  glutIdleFunc(idle);
  

  //Cam
  cam.to = Vector(0, 15, 5);
  cam.ang = Vector(3, 42, 155);
  cam.fov = 35.0;
  update_cam();

  const Vector &b = domain_b.bottom(),
               &t = domain_b.top();

  xh.resize(2);
  h.resize(2);

  xh[0] = b.x;
  xh[1] = t.x;
}

int main(int argc, char *argv[])
{
  glutInit(&argc, &argv[0]);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize(WIDTH, HEIGHT);
  glutInitWindowPosition(100, 100);
  winid = glutCreateWindow("fluid simulation");

  init();

  if(argc == 2)
    max_time = atof(argv[1]);
  else {
    std::cout << "Durée de la simulation (en secondes) : ";
    std::cin >> max_time;
  }

  std::cout << std::endl << "Let's go !" << std::endl;

  speed_theo_plot.setMax(Vector(max_time, 0.7*max_time * 3.6)); //I guess max acc is 2 m/s^2
  speed_actual_plot.setMax(Vector(max_time, 0.7*max_time * 3.6)); //I guess max acc is 2 m/s^2
  eta_plot[0].setMax(Vector(max_time, 2 * fluid_b.top().z * sph.getParams().simscale));
  eta_plot[1].setMax(Vector(max_time, 2 * fluid_b.top().z * sph.getParams().simscale));

  glutMainLoop();

  srand(time(NULL));

  return 0;
}
